@@ -1,26 +1,59 @@
 """Layers"""
+
+import io
+import json
 import os
 import tempfile
+import typing
+import urllib.request
+import uuid
 
-import requests
+from urllib.parse import urljoin
 
-from .api import (
-    make_request,
-    LAYERS_TEMPLATE,
-    REFRESH_TEMPLATE,
-    UPDATE_STYLE_TEMPLATE,
-    UPLOAD_TEMPLATE,
-)
+from .api import make_request, BASE_URL
+from .maps import MAP_TEMPLATE
+
+
+MAP_LAYERS_TEMPLATE = urljoin(BASE_URL, "maps/{map_id}/layers/")
+LAYER_TEMPLATE = urljoin(MAP_LAYERS_TEMPLATE, "{layer_id}/")
+REFRESH_TEMPLATE = urljoin(LAYER_TEMPLATE, "refresh")
+UPDATE_STYLE_TEMPLATE = urljoin(LAYER_TEMPLATE, "update_style")
+UPLOAD_TEMPLATE = urljoin(MAP_TEMPLATE, "upload")
 
 
 def list_layers(map_id: str, api_token: str | None = None):
     """List layers on a map"""
     response = make_request(
-        url=LAYERS_TEMPLATE.expand(map_id=map_id),
-        method=requests.get,
+        url=MAP_LAYERS_TEMPLATE.format(map_id=map_id),
+        method="GET",
         api_token=api_token,
     )
-    return response.json()["data"]
+    return json.load(response)
+
+
+def _multipart_request(
+    url: str, presigned_attributes: dict[str, str], file_obj: typing.IO[bytes]
+) -> urllib.request.Request:
+    """Make a multipart/form-data request with the given file"""
+    boundary = "-" * 20 + str(uuid.uuid4())
+    headers = {"Content-Type": f'multipart/form-data; boundary="{boundary}"'}
+    fname = os.path.basename(file_obj.name)
+
+    data = io.BytesIO()
+    text = io.TextIOWrapper(data, encoding="latin-1")
+    for key, value in presigned_attributes.items():
+        text.write(f"--{boundary}\r\n")
+        text.write(f'Content-Disposition: form-data; name="{key}"\r\n\r\n')
+        text.write(f"{value}\r\n")
+    text.write(f"--{boundary}\r\n")
+    text.write(f'Content-Disposition: form-data; name="file"; filename="{fname}"\r\n')
+    text.write("Content-Type: application/octet-stream\r\n\r\n")
+    text.flush()
+    data.write(file_obj.read())
+    data.write(f"\r\n--{boundary}".encode("latin-1"))
+    body = data.getvalue()
+
+    return urllib.request.Request(url, data=body, headers=headers, method="POST")
 
 
 def _request_and_upload(
@@ -36,22 +69,16 @@ def _request_and_upload(
     requires a layer name while the refresh endpoint does not.
     """
     json_payload = {"name": layer_name} if layer_name else None
-    layer_response = make_request(
-        url=url, method=requests.post, api_token=api_token, json=json_payload
+    response = make_request(
+        url=url, method="POST", api_token=api_token, json=json_payload
     )
-    presigned_upload = layer_response.json()
-
-    url = presigned_upload["data"]["attributes"]["url"]
-    presigned_attributes = presigned_upload["data"]["attributes"][
-        "presigned_attributes"
-    ]
+    presigned_upload = json.load(response)
+    url = presigned_upload["url"]
+    presigned_attributes = presigned_upload["presigned_attributes"]
     with open(file_name, "rb") as file_obj:
-        requests.post(
-            url,
-            # Order is important, file should come at the end
-            files={**presigned_attributes, "file": file_obj},
-        )
-    return layer_response.json()["data"]
+        request = _multipart_request(url, presigned_attributes, file_obj)
+        urllib.request.urlopen(request)
+    return presigned_upload
 
 
 def upload_file(
@@ -62,7 +89,7 @@ def upload_file(
 ):
     """Upload a file to a Felt map"""
     return _request_and_upload(
-        url=UPLOAD_TEMPLATE.expand(map_id=map_id),
+        url=UPLOAD_TEMPLATE.format(map_id=map_id),
         file_name=file_name,
         layer_name=layer_name,
         api_token=api_token,
@@ -110,7 +137,7 @@ def refresh_file_layer(
 ):
     """Refresh a layer originated from a file upload"""
     return _request_and_upload(
-        url=REFRESH_TEMPLATE.expand(map_id=map_id, layer_id=layer_id),
+        url=REFRESH_TEMPLATE.format(map_id=map_id, layer_id=layer_id),
         file_name=file_name,
         api_token=api_token,
     )
@@ -123,29 +150,29 @@ def upload_url(
     api_token: str | None = None,
 ):
     """Upload a URL to a Felt map"""
-    layer_response = make_request(
-        url=UPLOAD_TEMPLATE.expand(map_id=map_id),
-        method=requests.post,
+    response = make_request(
+        url=UPLOAD_TEMPLATE.format(map_id=map_id),
+        method="POST",
         api_token=api_token,
         json={
             "import_url": layer_url,
             "name": layer_name,
         },
     )
-    return layer_response.json()["data"]
+    return json.load(response)
 
 
 def refresh_url_layer(map_id: str, layer_id: str, api_token: str | None = None):
     """Refresh a layer originated from a URL upload"""
-    layer_response = make_request(
-        url=REFRESH_TEMPLATE.expand(
+    response = make_request(
+        url=REFRESH_TEMPLATE.format(
             map_id=map_id,
             layer_id=layer_id,
         ),
-        method=requests.post,
+        method="POST",
         api_token=api_token,
     )
-    return layer_response.json()["data"]
+    return json.load(response)
 
 
 def get_layer_details(
@@ -155,14 +182,14 @@ def get_layer_details(
 ):
     """Get style of a layer"""
     response = make_request(
-        url=LAYERS_TEMPLATE.expand(
+        url=LAYER_TEMPLATE.format(
             map_id=map_id,
             layer_id=layer_id,
         ),
-        method=requests.get,
+        method="GET",
         api_token=api_token,
     )
-    return response.json()["data"]
+    return json.load(response)
 
 
 def update_layer_style(
@@ -173,12 +200,12 @@ def update_layer_style(
 ):
     """Style a layer"""
     response = make_request(
-        url=UPDATE_STYLE_TEMPLATE.expand(
+        url=UPDATE_STYLE_TEMPLATE.format(
             map_id=map_id,
             layer_id=layer_id,
         ),
-        method=requests.post,
+        method="POST",
         json={"style": style},
         api_token=api_token,
     )
-    return response.json()["data"]
+    return json.load(response)
